"""
"""


class BlyncLightException(Exception):
    pass


class BlyncLightNotFound(BlyncLightException):
    pass


class BlyncLightInUse(BlyncLightException):
    pass


class BlyncLightUnknownDevice(BlyncLightException):
    pass



"""
"""
from typing import List, Tuple
import math


def Gradient(
    start: int, stop: int, step: int, red=True, green=False, blue=False
) -> List[Tuple[int, int, int]]:
    """
    """
    colors = []
    for i in range(start, stop, step):
        colors.append((i if red else 0, i if blue else 0, i if green else 0))
    return colors


def Spectrum(
    steps: int = 64,
    frequency: Tuple[float, float, float] = None,
    phase: Tuple[int, int, int] = None,
    center: int = 128,
    width: int = 127,
) -> Tuple[float, float, float]:
    """Generator function that returns 'steps' (red,green,blue) tuples.

        steps: optional integer, default=64
    frequency: optional 3-tuple for rbg frequency, default=(.3,.3,.3)
        phase: optional 3-tuple for rbg phase, default=(0,2,4)
       center: optional integer, default=128
        width: optional integer, default=127

    Returns (r, b, g) where each member is a value between 0 and 255.
    """

    frequency = frequency or (0.3, 0.3, 0.3)
    phase = phase or (0, 2, 4)

    for i in range(steps):
        r = int((math.sin(frequency[0] * i + phase[0]) * width) + center)
        b = int((math.sin(frequency[2] * i + phase[2]) * width) + center)
        g = int((math.sin(frequency[1] * i + phase[1]) * width) + center)
        yield (r, b, g)



"""Embrava Device Constants
"""

import enum

EMBRAVA_VENDOR_IDS = [0x2C0D, 0x0E53]
END_OF_COMMAND = 0xFF22
COMMAND_LENGTH = 9
PAD_VALUE = 0


class DeviceType(enum.Enum):
    INVALID = 0
    TENX_10 = 1
    TENX_20 = 2
    V30 = 3
    V30S = 4
    V30_LUMENA110 = 5
    WIRELESS_V30S = 6
    MINI_V30S = 7
    V30_LUMENA120 = 8
    V30_LUMENA = 9
    V30_LUMENA210 = 10
    V30_LUMENA220 = 11
    EMBEDDED_V30 = 12


class FlashSpeed(enum.IntEnum):
    OFF = 0
    LOW = 1
    MEDIUM = 2
    HIGH = 4

    def is_valid(self, speed):
        """Returns True if speed is a valid flash speed value.

        """
        return speed in [self.OFF, self.LOW, self.MEDIUM, self.HIGH]


class MusicSelections(enum.Enum):
    pass


class MusicVolume(enum.IntEnum):
    Vol010Percent = 1
    Vol020Percent = 2
    Vol030Percent = 3
    Vol040Percent = 4
    Vol050Percent = 5
    Vol060Percent = 6
    Vol070Percent = 7
    Vol080Percent = 8
    Vol090Percent = 9
    Vol100Percent = 10



"""bindings for Embrava BlyncLight devices.

This module provides python bindings for interacting with the Embrava
BlyncLight family of devices via Human Input Device Application
Programming Interface (HIDAPI). HIDAPI is a C language library that
provides access to USB devices.

You can control Embrava BlyncLight devices directly:

> from blynclight import BlyncLight
> b = BlyncLight.get_light()
> b.color = (0, 255, 0)                         # set color to blue
> b.on = True                                   # turn light on
> b.color = (255, 0, 0)                         # set color to red
> b.on = False                                  # turn light off

Or control the device via http API:

> from blynclight import BlyncLightProxy
> proxy = BlyncLightProxy('http://yourhost:port')
> available = proxy.status()
> proxy.on(0)
> proxy.red(0, 255)
> proxy.green(0, 0)
> proxy.blue(0, 255)
> proxy.color(0, (255,255,255))
> proxy.off(0)

The BlyncLightProxy method of control requires a HTTP server that
provides the expected REST interface. This is somewhat more involved
than talking to the light directly, but allows more flexibility.
"""

from .blynclight import BlyncLight

from .constants import FlashSpeed, MusicSelections, MusicVolume

from .exceptions import BlyncLightNotFound, BlyncLightUnknownDevice, BlyncLightInUse

from .proxy import Proxy as BlyncLightProxy


__all__ = [
    "BlyncLight",
    "BlyncLightProxy",
    "BlyncLightNotFound",
    "BlyncLightInUse",
    "BlyncLightUnknownDevice",
    "FlashSpeed",
    "MusicSelections",
    "MusicVolume",
]



"""BlyncLight CLI Utility

Control your Embrava BlyncLight from the command-line!

"""

import typer


from blynclight import BlyncLight, BlyncLightNotFound
from collections import deque
from itertools import cycle

from pprint import pprint
from sys import stdout
from time import sleep

from .effects import Gradient, Spectrum
from .__version__ import __version__

cli = typer.Typer()

DEFAULT_COLOR = (0, 0, 255)  # (Red, Blue, Green)


def list_lights(value: bool) -> None:
    """Display a list of BlyncLights currently available and exit.

    Typer option callback.
    """
    if value:
        typer.secho(f"{'BlyncLights':19s}:", nl=False)
        lights = BlyncLight.available_lights()
        nlights = len(lights)
        typer.secho(f"{nlights}", fg="green" if nlights else "red")
        for index, info in enumerate(lights):
            typer.secho(f"ID:{'KEY':<16s}:VALUE", fg="blue")
            for key, value in info.items():
                if not len(str(value)) or key == "path":
                    continue
                if isinstance(value, int):
                    value = hex(value)
                if isinstance(value, bytes):
                    value = value.decode("utf-8")
                typer.secho(f"{index:02d}:{key:<16s}:", nl=False)
                typer.secho(value, fg="green")
        raise typer.Exit()


def report_version(value: bool) -> None:
    """Display the version and exit.

    Typer option callback.
    """
    if value:
        print(f"version: {__version__}")
        raise typer.Exit()


@cli.callback(invoke_without_command=True)
def blync_callback(
    ctx: typer.Context,
    light_id: int = typer.Option(
        0, "--light-id", "-l", show_default=True, help="Light identifier",
    ),
    red: int = typer.Option(
        0,
        "--red",
        "-r",
        is_flag=True,
        show_default=True,
        help="Red color value range: 0 - 255",
    ),
    blue: int = typer.Option(
        0,
        "--blue",
        "-b",
        is_flag=True,
        show_default=True,
        help="Blue color value range: 0 - 255",
    ),
    green: int = typer.Option(
        0,
        "--green",
        "-g",
        is_flag=True,
        help="Green color value range: 0 - 255",
        show_default=True,
    ),
    red_b: bool = typer.Option(
        False, "--RED", "-R", is_flag=True, help="Full value red [255]"
    ),
    blue_b: bool = typer.Option(
        False, "--BLUE", "-B", is_flag=True, help="Full value blue [255]"
    ),
    green_b: bool = typer.Option(
        False, "--GREEN", "-G", is_flag=True, help="Full value green [255]"
    ),
    off: bool = typer.Option(
        False, "--off/--on", "-o/-n", show_default=True, help="Turn the light off/on."
    ),
    dim: bool = typer.Option(
        False,
        "--dim",
        "-d",
        is_flag=True,
        help="Toggle bright/dim mode.",
        show_default=True,
    ),
    flash: int = typer.Option(
        0, "--flash", "-f", count=True, is_flag=True, help="Enable flash mode.",
    ),
    play: int = typer.Option(0, "--play", "-p", help="Select song: 1-15"),
    repeat: bool = typer.Option(
        False,
        "--repeat",
        is_flag=True,
        show_default=True,
        help="Repeat the selected song.",
    ),
    volume: int = typer.Option(
        5, "--volume", show_default=True, help="Set the volume: 1-10"
    ),
    available: bool = typer.Option(
        False,
        "--list-available",
        "-a",
        is_flag=True,
        is_eager=True,
        callback=list_lights,
    ),
    verbose: int = typer.Option(0, "--verbose", "-v", count=True),
    version: bool = typer.Option(
        False, "--version", "-V", is_flag=True, is_eager=True, callback=report_version
    ),
):
    """Control your Embrava BlyncLight from the command-line!

    Use the `blync` utility to directly control your Embrava BlyncLight:

    \b
    ```console
    $ blync -R        # turn the light on with red color and leave it on
    $ blync --off     # turn the light off
    $ blync -RG --dim # turn the light on with yellow color and dim
    $ blync -RBG      # turn the light on with white color
    ```

    Colors can be specified by values between 0 and 255 using the lower-case
    color options or using the upper-case full value options.

    \b
    ```console
    $ blync -r 127                # half intensity red
    $ blync -r 255                # full intensity red
    $ blync -R                    # also full intensity red
    $ blync -r 255 -b 255 -g 255  # full intensity white
    $ blync -RBG                  # full intensity white
    ```
    

    If that's not enough fun, there are three builtin color modes:
    `fli`, `throbber`, and `rainbow`. All modes continue until the
    user terminates with a Control-C or platform equivalent.
    
    \b
    ```console
    $ blync fli
    $ blync throbber
    $ blync rainbow
    ```
    """

    try:
        light = BlyncLight.get_light(light_id, immediate=False)
    except Exception as error:
        typer.secho(str(error), fg="red")
        raise typer.Exit(-1)

    assert light.immediate == 0

    light.red = red if not red_b else 255
    light.blue = blue if not blue_b else 255
    light.green = green if not green_b else 255
    light.off = off
    light.dim = dim
    light.flash = flash > 0
    light.speed = 1 << (flash - 1) if flash else 0

    light.mute = 0 if play else 1
    light.music = play
    light.volume = volume
    light.repeat = repeat

    if not ctx.invoked_subcommand:
        if light.on and light.color == (0, 0, 0):
            light.color = DEFAULT_COLOR
        light.immediate = 1
        if verbose:
            print(light)
        typer.Exit()

    # Disable flashing for subcommands.
    light.flash = 0
    light.speed = 0

    ctx.obj = light


@cli.command("fli")
def fli_subcommand(
    ctx: typer.Context,
    interval: float = typer.Option(
        0.1, "--interval", "-n", help="Seconds between flashes.", show_default=True,
    ),
    intensity: int = typer.Option(
        255, "--intensity", "-i", help="Integer range: 0 - 255", show_default=True,
    ),
):
    """Flash Light Impressively.

    This mode cycles light color red, blue, green and then repeats. The
    user can specify the interval between color changes and the intesity
    of the colors. Color values specified on the command-line are ignored.

    \b
    ```console
    $ blync fli -n 1      # one second between color changes
    $ blync fli -i 128    # light intensity is half as bright
    ```
    
    This mode runs until the user interrupts.
    """

    light = ctx.obj

    color = deque([(0x0FF & intensity) >> 0, 0, 0])

    try:
        light.color = color
        light.on = True
        light.immediate = 1

        while True:
            color.rotate(1)
            light.color = color
            sleep(interval)

    except KeyboardInterrupt:
        light.off = True
        light.reset()


@cli.command("throbber")
def throbber_subcommand(
    ctx: typer.Context,
    fast: int = typer.Option(0, "--faster", "-f", help="Increases speed.", count=True,),
):
    """BlyncLight Intensifies.

    This mode increases the intensity of the light color starting with
    the specified red, green and blue values and ramping the color
    intensity up and down and repeating. The user can increase the rate
    of ramp by adding more -f options to the command line:

    \b
    ```console
    $ blync throbber -f   # a little faster
    $ blync throbber -ff  # a little more faster
    $ blync -G throbber   # throb with a green color
    ```
    
    This mode runs until the user interrupts.
    """

    light = ctx.obj

    if light.color == (0, 0, 0):
        light.red = 255

    try:
        step = 8 * (min(max(0, fast), 24) + 1)
        colors = Gradient(0, 255, step, light.red, light.green, light.blue)
        colors += reversed(colors)

        light.color = (0, 0, 0)
        light.immediate = 1

        for color in cycle(colors):
            light.color = color
            sleep(0.05)

    except KeyboardInterrupt:
        light.off = True
        light.reset()


@cli.command("rainbow")
def rainbow_subcommand(
    ctx: typer.Context,
    speed: int = typer.Option(
        1,
        "--slow",
        "-s",
        help="Increase color cycle interval by 0.1 seconds.",
        is_flag=True,
        count=True,
    ),
):
    """BlyncLights Love Rainbows!

    Smoothly transition the color of the light using a rainbow sequence.
    The user can slow the speed of the color cycling by adding more
    --slow options to the command line:

    \b
    ```console
    $ blync rainbow -s   # slow cycling by 0.1 seconds
    $ blync rainbow -ss  # slow cycling by 0.15 seconds
    ```

    This mode runs until the user interrupts.
    """

    light = ctx.obj

    try:
        colors = [rgb for rgb in Spectrum(steps=255)]
        interval = speed * 0.05

        light.on = True
        light.color = (0, 0, 0)
        light.immediate = 1

        for color in cycle(colors):
            light.color = color
            sleep(interval)

    except KeyboardInterrupt:
        light.off = True
        light.reset()



"""Embrava Blynclight Support
"""

from contextlib import contextmanager
from ctypes import Structure, c_uint64
from typing import Any, Dict, List, Tuple, Union

from .constants import EMBRAVA_VENDOR_IDS, END_OF_COMMAND, COMMAND_LENGTH, PAD_VALUE
from .exceptions import BlyncLightNotFound, BlyncLightUnknownDevice, BlyncLightInUse
from .hid import HidDevice


class BlyncLight(Structure):
    """BlyncLight

    The Embrava BlyncLight family of USB connected products responds
    to a 9-byte command word. The command word enables activation or
    deactivation of various features; turning the light on and off,
    changing it's color, causing the light to flash, dimming or
    brightening, playing musical tunes stored in some BlyncLight
    model's firmware, muting or changing the volume of the playing
    music.

    Not all devices have musical capability and music related
    functionality has not yet been tested. 3 Apr 2019

    The recommended way to obtain a BlyncLight object is to use the
    class method get_light().

    >>> light = BlyncLight.get_light()

    Features of the light are controlled by assigning values to
    attributes of the BlyncLight object:

    >>> light.red = 0xff
    >>> light.on = 1
    >>> light.on = 0

    Usage Notes
    ===========

    The BlyncLight object is an in-memory representation of the
    state of the hardware device. I have not been able to discern how
    to read the device's current state, so every time a new object is
    instantiated it writes a known state to the device by default:
    all fields zeros except off.

    Additionally, any updates to the bit fields in the ByncLight class
    will be immediately written to the hardware device by default.

    Callers can defer hardware updates by setting the 'immediate'
    attribute to False. Any changes to command fields will not be
    written to the device. Setting 'immediate' to True will write the
    current state of the command word to the device. The BlyncLight
    object also provides a context manager method that can be used to
    defer updates:

    >>> with light.updates_paused():
    ...     light.red = 0
    ...     light.blue = 255
    ...     light.green = 0
    ...     light.on = True

    is equivalent to:

    >>> light.immediate = False
    >>> light.red = 0
    >>> light.blue = 255
    >>> light.green = 0
    >>> light.on = True
    >>> light.immediate = True

    is equivalent (almost) to:

    >>> light.color = (0, 255, 0)
    >>> light.on = True

    ===CAVEAT===

    Before turning the light on, make sure to specify a color
    otherwise the device will not emit any light. It can be
    very frustrating to turn the light on and not have any
    noticible effect.

    Command Word Documentation
    ==========================


    The Embrava BlyncLight command word bit fields are:

    report : 8     Must be zero
    red    : 8     Red component value between 0-255
    blue   : 8     Blue component value between 0-255
    green  : 8     Green component value between 0-255
    off    : 1     0==on     1==off
    dim    : 1     0==bright 1==dim
    flash  : 1     0==steady 1==flash
    speed  : 3     0==off 1==low 2==medium 4==fast
    pad    : 2
    mute   : 1     0==unmute 1==mute
    music  : 4     select a built-in musical tune
    play   : 1     0==stop 1==play
    repeat : 1     0==no repeat 1==repeat
    pad    : 2
    volume : 4     1-10, increase volume by 10% per increment
    pad    : 2
    eoc    : 16    End Of Command field, must be 0xffff

    """

    _fields_ = [
        ("report", c_uint64, 8),
        ("red", c_uint64, 8),
        ("blue", c_uint64, 8),
        ("green", c_uint64, 8),
        ("off", c_uint64, 1),
        ("dim", c_uint64, 1),
        ("flash", c_uint64, 1),
        ("speed", c_uint64, 3),
        ("pad0", c_uint64, 2),
        ("music", c_uint64, 4),
        ("play", c_uint64, 1),
        ("repeat", c_uint64, 1),
        ("pad1", c_uint64, 2),
        ("volume", c_uint64, 4),
        ("pad2", c_uint64, 3),
        ("mute", c_uint64, 1),
        ("eoc", c_uint64, 16),
        # End of Command Word
        # - pad3 and immediate round out the
        #   length of _fields_ to 128 bits
        ("pad3", c_uint64, 55),
        ("immediate", c_uint64, 1),
    ]

    _ignore_ = ("report", "pad", "eoc")  # fields that are *not* user commands

    @classmethod
    def available_lights(cls) -> List[Dict[str, Any]]:
        """:return: list of dictionaries

        Returns a list of dictonary entries, each entry describing an
        Embrava BlyncLight device. If no matching devices are found,
        an empty list is returned.

        Raises:

        - KeyError if vendor_id key is missing in DeviceInfo dictionaries.

        """

        is_blynclight = lambda d: d["vendor_id"] in EMBRAVA_VENDOR_IDS

        return [info for info in HidDevice.enumerate() if is_blynclight(info)]

    @classmethod
    def get_light(cls, light_id: int = 0, immediate: bool = True):
        """:param light_id: optional integer
        :return: BlyncLight

        Returns a BlyncLight object accessed by index 'light_id' into
        a list returned by available_lights().

        - BlyncLightNotFound raised if light_id is not found.
        - BlyncLightInUse raised if BlyncLight has already been opened.

        """
        try:
            return cls.from_dict(cls.available_lights()[light_id], immediate=immediate)
        except IndexError:
            raise BlyncLightNotFound(f"Light for {light_id} not found.") from None

    @classmethod
    def from_dict(cls, info: Dict[str, int], immediate: bool = True):
        """:param info: dictionary
        :return: BlyncLight

        Returns a BlyncLight configured with the contents of the
        supplied dictionary 'info'. The keys 'vendor_id' and
        'product_id' are required.

        - BlyncLightInUse raised if specified light is already open.
        - KeyError if vendor_id or product_id is missing from the
          input dictionary.

        """
        return cls(info["vendor_id"], info["product_id"], immediate=immediate)

    def __init__(
        self, vendor_id: int, product_id: int, immediate: bool = True,
    ):
        """:param vendor_id:  16-bit integer
        :param product_id: 16-bit integer
        :param immediate:  optional boolean

        Initialize a BlyncLight for use.

        The vendor_id and product_id together specify a unique USB
        device. The immediate argument configures whether or not the
        object will immediately update the physical device with the
        object's in-memory representation of the command word.

        If immediate is False, the caller will need to set immediate
        to True before updates to the device will occur.

        The following exceptions are raised:

        - BlyncLightUnknownDevice is raised if vendor_id does not
          match known Embrava vendor indentifiers.

        - BlyncLightInUse is raised if the light specified by
          vendor_id:product_id has already been opened.

        - BlyncLightNotFound is raised if the light specified by
          vendor_id:product_id does not exist.

        """
        if vendor_id not in EMBRAVA_VENDOR_IDS:
            raise BlyncLightUnknownDevice(f"Unknown vendor id: 0x{vendor_id:04}")
        self.vendor_id = vendor_id
        self.product_id = product_id
        self.reset(flush=False)
        self.immediate = immediate

    def reset(self, flush: bool = True) -> None:
        """:param flush: optional boolean
        :return: None

        Returns the command word to it's default state and writes the
        command to the device if flush is True.

        On return the immediate attribute is zero.
        """
        self.immediate = 0
        for name, typ, sz in self._fields_:
            setattr(self, name, 0)
        self.off = 1
        if flush:
            self.device.write(self.bytes)

    def __str__(self) -> str:
        s = []
        for name, value in self.status.items():
            s.append(f"{self.device.identifier}:{name:>9s}:0x{value:04x}")
        return "\n".join(s)

    @property
    def status(self) -> Dict[str, int]:
        """A dictionary representation of the current light status.
        The keys are the command field names and the values are the
        integer contents of those fields.
        """
        retval = {}
        for command in self.commands:
            value = getattr(self, command)
            retval.setdefault(command, value)
        return retval

    def __len__(self) -> int:
        """The length of the command word in bytes.
        """
        return COMMAND_LENGTH

    def __del__(self) -> None:

        self.device.close()

    def __setattr__(self, name: str, value: Any) -> None:
        """Setting any BlyncLight command attribute triggers a write
        to the device if the immediate attribute is 1. If immediate
        is 0, the in-memory representation is changed but the
        hardware is not updated with the new values. Setting
        immediate to 1 will flush the in-memory command word
        to the hardware.
        """

        if name.startswith(("report", "pad")):
            super().__setattr__(name, PAD_VALUE)
            return

        if name == "eoc":
            super().__setattr__(name, END_OF_COMMAND)
            return

        super().__setattr__(name, value)

        if name in self.commands and self.immediate:
            n = self.device.write(self.bytes)
            if n != len(self.bytes):
                raise IOError(f"Wrote {n} bytes, expected {len(self.bytes)} bytes")

    @property
    def device(self) -> HidDevice:
        """A blynclight.hid.HidDevice providing write access to
        an Embrava BlyncLight device.
        """
        try:
            return self._device
        except AttributeError:
            pass

        try:
            self._device = HidDevice(self.vendor_id, self.product_id)
        except ValueError:
            raise BlyncLightInUse(self.vendor_id, self.product_id)
        except LookupError:
            raise BlyncLightNotFound(self.vendor_id, self.product_id)

        return self._device

    @property
    def bytes(self) -> bytes:
        """A bytes representation of the 9 byte command word.
        """
        return bytes(self)[: len(self)]

    @property
    def commands(self) -> list:
        """List of valid BlyncLight command fields.
        """
        try:
            return self._commands
        except AttributeError:
            pass
        self._commands = [
            f for f, t, s in self._fields_ if not f.startswith(self._ignore_)
        ]
        return self._commands

    @contextmanager
    def updates_paused(self) -> None:
        """Context manager that suspends immediate updating of the device
        for the duration of the manager's execution. When the manager exits
        the immediate bit is reset to it's previous value, which may trigger
        a write to the hardware.
        """
        imm = self.immediate
        self.immediate = 0
        try:
            yield
        finally:
            self.immediate = imm

    @property
    def color(self) -> Tuple[int]:
        """A tuple of (red, blue, green) hexadecimal values.

        A three-tuple of 8-bit integers or 24-bit hex number can be
        used to set all three colors at once.

        > red, blue, green = light.colors
        > light.colors = (0xRR, 0xBB, 0xGG)
        > light.colors = 0xRRBBGG

        """
        return (self.red, self.blue, self.green)

    @color.setter
    def color(self, newValue: Union[int, Tuple[int]]):
        with self.updates_paused():
            try:
                self.red = (newValue >> 16) & 0x00FF
                self.blue = (newValue >> 8) & 0x00FF
                self.green = newValue & 0x00FF
                return
            except TypeError:
                pass
            self.red, self.blue, self.green = newValue

    @property
    def on(self) -> bool:
        """Inverse logic setter/getter for 'off' attribute.
        """
        return bool(0 if self.off else 1)

    @on.setter
    def on(self, newValue: bool) -> None:
        self.off = 0 if newValue else 1

    @property
    def bright(self) -> bool:
        """Inverse logic setter/getter for 'dim' attribute.
        """
        return bool(0 if self.dim else 1)

    @bright.setter
    def bright(self, newValue: bool) -> None:
        self.dim = 0 if newValue else 1



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['blynclight', 'blynclight.hid']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.0,<8.0', 'requests>=2.21,<3.0', 'typer>=0.2.1,<0.3.0']

entry_points = \
{'console_scripts': ['blync = blynclight.__main__:cli']}

setup_kwargs = {
    'name': 'blynclight',
    'version': '0.6.4',
    'description': 'Python language bindings for Embrava BlyncLight devices.',
    'long_description': "# `blynclight`\n\nControl your Embrava BlyncLight from the command-line!\n\nUse the `blync` utility to directly control your Embrava BlyncLight:\n\n\n```console\n$ blync -R        # turn the light on with red color and leave it on\n$ blync --off     # turn the light off\n$ blync -RG --dim # turn the light on with yellow color and dim\n$ blync -RBG      # turn the light on with white color\n```\n\nColors can be specified by values between 0 and 255 using the lower-case\ncolor options or using the upper-case full value options.\n\n\n```console\n$ blync -r 127                # half intensity red\n$ blync -r 255                # full intensity red\n$ blync -R                    # also full intensity red\n$ blync -r 255 -b 255 -g 255  # full intensity white\n$ blync -RBG                  # full intensity white\n```\n\n\nIf that's not enough fun, there are three builtin color modes:\n`fli`, `throbber`, and `rainbow`. All modes continue until the\nuser terminates with a Control-C or platform equivalent.\n\n\n```console\n$ blync fli\n$ blync throbber\n$ blync rainbow\n```\n\n**Usage**:\n\n```console\n$ blynclight [OPTIONS] COMMAND [ARGS]...\n```\n\n**Options**:\n\n* `-l, --light-id INTEGER`: Light identifier  [default: 0]\n* `-r, --red INTEGER`: Red color value range: 0 - 255  [default: 0]\n* `-b, --blue INTEGER`: Blue color value range: 0 - 255  [default: 0]\n* `-g, --green INTEGER`: Green color value range: 0 - 255  [default: 0]\n* `-R, --RED`: Full value red [255]\n* `-B, --BLUE`: Full value blue [255]\n* `-G, --GREEN`: Full value green [255]\n* `-o, --off / -n, --on`: Turn the light off/on.  [default: False]\n* `-d, --dim`: Toggle bright/dim mode.  [default: False]\n* `-f, --flash`: Enable flash mode.\n* `-p, --play INTEGER`: Select song: 1-15\n* `--repeat`: Repeat the selected song.  [default: False]\n* `--volume INTEGER`: Set the volume: 1-10  [default: 5]\n* `-a, --list-available`\n* `-v, --verbose`\n* `-V, --version`\n* `--install-completion`: Install completion for the current shell.\n* `--show-completion`: Show completion for the current shell, to copy it or customize the installation.\n* `--help`: Show this message and exit.\n\n**Commands**:\n\n* `fli`: Flash Light Impressively.\n* `rainbow`: BlyncLights Love Rainbows! Smoothly...\n* `throbber`: BlyncLight Intensifies.\n\n## `blynclight fli`\n\nFlash Light Impressively.\n\nThis mode cycles light color red, blue, green and then repeats. The\nuser can specify the interval between color changes and the intesity\nof the colors. Color values specified on the command-line are ignored.\n\n\n```console\n$ blync fli -n 1      # one second between color changes\n$ blync fli -i 128    # light intensity is half as bright\n```\n\nThis mode runs until the user interrupts.\n\n**Usage**:\n\n```console\n$ blynclight fli [OPTIONS]\n```\n\n**Options**:\n\n* `-n, --interval FLOAT`: Seconds between flashes.  [default: 0.1]\n* `-i, --intensity INTEGER`: Integer range: 0 - 255  [default: 255]\n* `--help`: Show this message and exit.\n\n## `blynclight rainbow`\n\nBlyncLights Love Rainbows!\n\nSmoothly transition the color of the light using a rainbow sequence.\nThe user can slow the speed of the color cycling by adding more\n--slow options to the command line:\n\n\n```console\n$ blync rainbow -s   # slow cycling by 0.1 seconds\n$ blync rainbow -ss  # slow cycling by 0.15 seconds\n```\n\nThis mode runs until the user interrupts.\n\n**Usage**:\n\n```console\n$ blynclight rainbow [OPTIONS]\n```\n\n**Options**:\n\n* `-s, --slow`: Increase color cycle interval by 0.1 seconds.\n* `--help`: Show this message and exit.\n\n## `blynclight throbber`\n\nBlyncLight Intensifies.\n\nThis mode increases the intensity of the light color starting with\nthe specified red, green and blue values and ramping the color\nintensity up and down and repeating. The user can increase the rate\nof ramp by adding more -f options to the command line:\n\n\n```console\n$ blync throbber -f   # a little faster\n$ blync throbber -ff  # a little more faster\n$ blync -G throbber   # throb with a green color\n```\n\nThis mode runs until the user interrupts.\n\n**Usage**:\n\n```console\n$ blynclight throbber [OPTIONS]\n```\n\n**Options**:\n\n* `-f, --faster`: Increases speed.\n* `--help`: Show this message and exit.\n",
    'author': "Erik O'Shaughnessy",
    'author_email': 'erik.oshaughnessy@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/JnyJny/blynclight.git',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
